package redirect

import (
	"errors"
	"log/slog"
	"net/http"
	resp "url-shortener/internal/lib/api/response"
	"url-shortener/internal/lib/logger/sl"
	"url-shortener/internal/storage"

	"github.com/go-chi/chi/v5"
	"github.com/go-chi/chi/v5/middleware"
	"github.com/go-chi/render"
)

//go:generate go run github.com/vektra/mockery/v2@v2.38.0 --name=URLProvider
type URLProvider interface {
	GetURL(alias string) (string, error)
}

func New(log *slog.Logger, urlProvider URLProvider) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		// setup logs
		const op = "handlers.url.redirect.New"

		log = log.With(
			slog.String("op", op),
			slog.String("request_id", middleware.GetReqID(r.Context())),
		)
		// get alias
		alias := chi.URLParam(r, "alias")
		if alias == "" {
			log.Info("alias is empty")
			render.JSON(w, r, resp.Error("invalid request"))
			return
		}
		// get url from storage
		url, err := urlProvider.GetURL(alias)
		if err != nil {
			if errors.Is(err, storage.ErrURLNotFound) {
				render.JSON(w, r, resp.Error("not found"))
				return
			}

			log.Error("failed to get url", sl.Err(err))
			render.JSON(w, r, resp.Error("internal error"))
			return
		}

		log.Info("got url", slog.String("url", url))

		// redirect to url
		http.Redirect(w, r, url, http.StatusFound)
	}
}
